//  Wii Remote IR sensor test  for ATMEGA168

#include <stdio.h>

#include "led_monitor.h"
#include "uart.h"
#include "i2c_master.h"

// i2c address
#define		I2C_ADR		0xB0		// ( 0x58 << 1 )

volatile int timer_val;
void wait(int tmax)
{
	int t; for (t=0;t<tmax;t++) { timer_val=t; }
}

int main()
{
	int i;
	int r;
	unsigned char data_buf[16];
	int p0,p1,p2,p3;
	int Ix1,Iy1,Ix2,Iy2;
	int Ix3,Iy3,Ix4,Iy4;
	int s;
	char buf[255];
	
	i2c_master_init();
	led_monitor_init(); // monitor LED setting
	
	//rs_init(57600);    // UART setting   57600bps
	//rs_init_by_BRR(8);   //  115200bps (16.000MHz Crystal for ATMEGA168)
	rs_init_by_BRR(9);   //  115200bps (18.342MHz Crystal ,  better than 16MHz)
	
	rs_puts("\n");
	rs_puts("Wii Remote IR sensor test\n");
	rs_puts("\n");
	
	led_monitor_on_off(1); // indicator LED on

    // IR sensor initialize
	
	//p0=0x72; p1=0x20; p2=0x1F; p3=0x03; // sensitivity
	//p0=0xC8; p1=0x36; p2=0x35; p3=0x03;
	  p0=0xAA; p1=0x64; p2=0x63; p3=0x03;
	//p0=0x96; p1=0xB4; p2=0xB3; p3=0x04;
	//p0=0x96; p1=0xFE; p2=0xFE; p3=0x05;

	r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
	i2c_write( 0x30 ); i2c_write( 0x01 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	wait(1000);
	
	r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
	i2c_write( 0x00 );
	i2c_write( 0x02 ); i2c_write( 0x00 ); i2c_write( 0x00 ); i2c_write( 0x71 );
	i2c_write( 0x01 ); i2c_write( 0x00 ); i2c_write( p0 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	wait(1000);

	r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
	i2c_write( 0x07 ); i2c_write( 0x00 ); i2c_write( p1 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	wait(1000);

	r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
	i2c_write( 0x1A ); i2c_write( p2 ); i2c_write( p3 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	wait(1000);

	r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
	i2c_write( 0x33 ); i2c_write( 0x03 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	wait(1000);

	r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
	i2c_write( 0x30 ); i2c_write( 0x08 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	wait(1000);


	led_monitor_on_off(0); // monitor LED off


	while (1) {
		
		led_monitor_on_off(1); // indicator LED on
		
		r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
		i2c_write( 0x36 );
		i2c_stop();
		if (r!=0) { rs_puts("Error: no ACK\n"); }
		
		r=i2c_start(I2C_ADR,I2C_READ_MODE);
		for (i=0;i<16;i++) { data_buf[i]=0; }
		for (i=0;i<16;i++) {
			if (i!=15) {
				data_buf[i] = i2c_read(I2C_READ_WITH_ACK);
			} else {
				data_buf[i] = i2c_read(I2C_READ_WITH_NACK);
			}
		}
		i2c_stop();
		if (r!=0) { rs_puts("Error: no ACK (2)\n"); }
		
		led_monitor_on_off(0); // LED off	
		
		Ix1 = data_buf[1]; Iy1 = data_buf[2]; s = data_buf[3];
		Ix1 += (s & 0x30) <<4; Iy1 += (s & 0xC0) <<2;
		Ix2 = data_buf[4]; Iy2 = data_buf[5]; s = data_buf[6];
		Ix2 += (s & 0x30) <<4; Iy2 += (s & 0xC0) <<2;
		Ix3 = data_buf[7]; Iy3 = data_buf[8]; s = data_buf[9];
		Ix3 += (s & 0x30) <<4; Iy3 += (s & 0xC0) <<2;
		Ix4 = data_buf[10]; Iy4 = data_buf[11]; s = data_buf[12];
		Ix4 += (s & 0x30) <<4; Iy4 += (s & 0xC0) <<2;
		
		sprintf(buf,"%d," ,Ix1); rs_puts(buf);
		sprintf(buf,"%d," ,Iy1); rs_puts(buf);
		sprintf(buf,"%d," ,Ix2); rs_puts(buf);
		sprintf(buf,"%d\n",Iy2); rs_puts(buf);
		
		wait(2000);
	}
	
	led_monitor_on_off(0); // LED off	
	return 0;
}
