#include <avr/io.h>
#include <avr/interrupt.h>
#include "uart.h"


// Clock Frequency for AVR
#define CLOCK 18432000UL


typedef unsigned char byte;
volatile byte uart_recv_buf[256]; // UART Receive buffer
volatile byte uart_recv_buf_write_pos,uart_recv_buf_read_pos;

ISR(USART_RX_vect)
{
	// UART receive interrupt  (data received)
	uart_recv_buf[uart_recv_buf_write_pos++] = UDR0; // receive to buffer
}

void rs_init_by_BRR(unsigned int brr)
{
	UBRR0H = (unsigned char)(brr>>8); UBRR0L = (unsigned char)brr;
	// stopbit 1bit , data=8bit
	UCSR0C = (3<<UCSZ00);

	// RX,TX enable  , RX interrupt enable
	UCSR0B = (1<<RXEN0) | (1<<TXEN0) | (1<<RXCIE0);
}

void rs_init(unsigned int baud)
{
	unsigned int brr; // baud rate
	brr = (CLOCK>>4)/baud - 1;
	rs_init_by_BRR(brr);
}

int rs_size(void)
{
	if (uart_recv_buf_write_pos ==  uart_recv_buf_read_pos) {
		return 0;
	}
	return 1;
}

int rs_getc()
{
	while (1) {
		if (rs_size()>0) { break; }
	}
    return uart_recv_buf[uart_recv_buf_read_pos++];
}

void rs_putc(int c)
{
	while (1) {
		if (UCSR0A & (1<<UDRE0)) { break; }	// possible to send?
	}
	UDR0 = c;
}

void rs_puts(char *s)
{
	while (*s) { rs_putc(*s++); }
}
